/*
* Copyright 2021-2023 H2020 TeraFlow (https://www.teraflow-h2020.eu/)
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*      http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package eu.teraflow.automation;

import static org.assertj.core.api.Assertions.assertThat;

import eu.teraflow.automation.context.model.GpsPosition;
import eu.teraflow.automation.context.model.LocationTypeGpsPosition;
import eu.teraflow.automation.context.model.LocationTypeRegion;
import io.quarkus.test.junit.QuarkusTest;
import org.junit.jupiter.api.Test;

@QuarkusTest
class LocationTypeTest {

    @Test
    void shouldExtractRegionFromLocationTypeRegion() {
        final var expectedRegion = "ATH";

        final var locationTypeRegion = new LocationTypeRegion(expectedRegion);

        assertThat(locationTypeRegion.getLocationType()).isEqualTo(expectedRegion);
    }

    @Test
    void shouldExtractLocationGpsPositionFromLocationTypeGpsPosition() {
        final var latitude = 3.99f;
        final var longitude = 77.32f;

        final var expectedLocationGpsPosition = new GpsPosition(latitude, longitude);
        final var locationTypeGpsPosition = new LocationTypeGpsPosition(expectedLocationGpsPosition);

        assertThat(locationTypeGpsPosition.getLocationType()).isEqualTo(expectedLocationGpsPosition);
    }
}
